"""Dump file utils collection"""

from typing import Any, Dict, Set, Union

from configs_core.base_setting_definition import BaseSettingDefinition, ExternalSettingDefinition
from configs_core.migration.consts import BLACK_LIST_LIMITATIONS


def generateDump(
    services: Set[str], settingsStore: Dict[str, Union[BaseSettingDefinition, ExternalSettingDefinition]]
) -> Dict[str, Any]:
    """
    Generate configurator dump for services.
    Args:
        settingsStore: settings store
        services: service name list

    Returns:
        dump
    """
    limitations = []
    settings = []
    groups = []
    for settingName, setting in settingsStore.items():
        if settingName in BLACK_LIST_LIMITATIONS:
            continue
        servicesIntersection = set(setting.services) & services
        if not servicesIntersection:
            continue
        if isinstance(setting, BaseSettingDefinition):
            limitation = {
                "limitation_name": settingName,
                "validation_schema": setting.schema,
                "services": sorted(list(servicesIntersection)),
                "default_value": setting.default,
                "description": setting.description,
                "group_names": setting.groups,
                "external": setting.external,
            }
            settingForLimitation = {
                "name": settingName,
                "value": setting.default,
                "description": setting.description,
                "tags": [],
            }
            settings.append(settingForLimitation)
        else:
            limitation = {
                "limitation_name": settingName,
                "validation_schema": setting.schema,
                "services": sorted(list(servicesIntersection)),
                "group_names": setting.groups,
                "external": setting.external,
                "default_value": setting.default,
            }
        groups.extend(setting.groups)
        limitations.append(limitation)

    return {
        "settings": sorted(settings, key=lambda setting: setting["name"]),
        "groups": [{"group_name": groupName, "description": ""} for groupName in sorted(set(groups))],
        "limitations": sorted(limitations, key=lambda limitation: limitation["limitation_name"]),
    }
